<?php
/*
 * Copyright (c) 2018 Methys Digital
 * All rights reserved.
 *
 * This software is the confidential and proprietary information of Methys Digital.
 * ("Confidential Information"). You shall not disclose such
 * Confidential Information and shall use it only in accordance with
 * the terms of the license agreement you entered into with Methys Digital.
 */
namespace Paygenius\Paygenius\Gateway\Http\Client;

use Magento\Checkout\Exception;
use Magento\Framework\Controller\ResultFactory;
use Magento\Payment\Gateway\Http\ClientInterface;
use Magento\Payment\Gateway\Http\TransferInterface;
use Magento\Payment\Model\Method\Logger;
use Psr\Log\LoggerInterface;
use Paygenius\Paygenius\Gateway\Intergration\CreditCard;
use Paygenius\Paygenius\Gateway\Intergration\Consumer;
use Paygenius\Paygenius\Gateway\Intergration\Transaction;
use Paygenius\Paygenius\Gateway\Intergration\CreatePaymentRequest;
use Paygenius\Paygenius\Gateway\Intergration\Client;
use Magento\Framework\Event\ManagerInterface;

class ClientMock implements ClientInterface
{
    const SUCCESS = 1;
    const FAILURE = 0;

    /**
     * @var array
     */
    private $results = [
        self::SUCCESS,
        self::FAILURE
    ];

    /**
     * @var Logger
     */
    private $logger;
    /**
     * @var LoggerInterface
     */
    private $loggerInterface;
    private $resultRedirect;
    /**
     * @var Client
     */
    private $client;
    /**
     * @var ManagerInterface
     */
    private $manager;
    /**
     * @var \Magento\Customer\Model\Session
     */
    private $customerSession;
    /**
     * @var \Magento\Framework\UrlInterface
     */
    private $urlInterface;
    /**
     * @var \Magento\Framework\Message\ManagerInterface
     */
    private $messageManager;

    /**
     * @param Logger $logger
     * @param LoggerInterface $loggerInterface
     * @param ResultFactory $result
     * @param ManagerInterface $manager
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Framework\UrlInterface $urlInterface
     * @param \Magento\Framework\Message\ManagerInterface $messageManager
     * @internal param Client $client
     */
    public function __construct(
        Logger $logger,
        LoggerInterface $loggerInterface,
        ResultFactory $result,
        ManagerInterface $manager,
        \Magento\Customer\Model\Session $customerSession,
        \Magento\Framework\UrlInterface $urlInterface,
        \Magento\Framework\Message\ManagerInterface $messageManager
    ) {
        $this->logger = $logger;
        $this->loggerInterface = $loggerInterface;
        $this->resultRedirect = $result;
        $this->manager = $manager;
        $this->customerSession = $customerSession;
        $this->urlInterface = $urlInterface;
        $this->messageManager = $messageManager;
    }

    /**
     * Places request to gateway. Returns result as ENV array
     *
     * @param TransferInterface $transferObject
     * @return array
     */
    public function placeRequest(TransferInterface $transferObject)
    {
        $response = $this->createPayment($transferObject);

        $this->logger->debug(
            [
                'request' => $transferObject->getBody(),
                'response' => $response
            ]
        );

        $this->loggerInterface->info("Client has concluded".json_encode($response));

        $this->handleResultCode($response->code??null);

        if ($response->code === 2)
        {
            $this->customerSession->setThreeDSecure(array_merge($this->handleThreed($response),["uri"=>$transferObject->getUri()]));

            return $this->generateResponseForCode(1, $this->handleThreed($response));
        }

        return $this->generateResponseForCode(0, null);
    }

    /**
     * Generates response
     *
     * @param $resultCode
     * @return array
     */
    protected function generateResponseForCode($resultCode)
    {

        return array_merge(
            [
                'RESULT_CODE' => $resultCode,
                'TXN_ID' => $this->generateTxnId()
            ],
            $this->getFieldsBasedOnResponseType($resultCode)
        );
    }

    /**
     * @return string
     */
    protected function generateTxnId()
    {
        return md5(mt_rand(0, 1000));
    }

    /**
     * Returns result code
     *
     * @param TransferInterface $transfer
     * @return int
     */
    private function getResultCode(TransferInterface $transfer)
    {
        $headers = $transfer->getHeaders();

        if (isset($headers['force_result'])) {
            return (int)$headers['force_result'];
        }

        return $this->results[mt_rand(0, 1)];
    }

    /**
     * Returns response fields for result code
     *
     * @param int $resultCode
     * @return array
     */
    private function getFieldsBasedOnResponseType($resultCode)
    {
        switch ($resultCode) {
            case self::FAILURE:
                return [
                    'FRAUD_MSG_LIST' => [
                        'Stolen card',
                        'Customer location differs'
                    ]
                ];
        }

        return [];
    }

    private function createPayment(TransferInterface $request)
    {
        $body = $request->getBody();

        $header = $request->getHeaders();

        $url = $request->getUri();

        $method = $request->getMethod();

        $creditCard = $this->initialiseCreditCard($body);

        $transaction = $this->initialiseTransaction($body);

        $consumer = $this->initialiseConsumer($body);

        $paymentRequest = new CreatePaymentRequest($creditCard, $transaction, true, $consumer);

        $paygeniusClient = new Client($body['token'], $body['key'], $url);

        return $paygeniusClient->createPayment($paymentRequest);

    }

    private function capturePayment(TransferInterface $request)
    {
        $body = $request->getBody();

        $url = $request->getUri();

        $paygeniusClient = new Client($body['token'], $body['key'], $url);

        return $paygeniusClient->executePayment($body['transaction']['reference']);
    }

    private function handleThreed($data)
    {
        return ['THREEDSECURE'=>[
            "acsUrl"=>$data->threeDSecure->acsUrl,
            "transactionId"=>$data->threeDSecure->transactionId,
            "paReq"=>$data->threeDSecure->paReq,
            "redirectUrl"=>$this->urlInterface->getBaseUrl()."paygeniusthreed",
            "reference"=>$data->reference
        ]];
    }

    private function handleResultCode($code)
    {
        switch ($code)
        {
            case 403 :

                $this->messageManager->addErrorMessage(_("Error processing the payment. Please try again or contact support."));

                throw new Exception(_("Error processing the payment. Please try again or contact support."));
                
                break;

            case 1000:

                $this->messageManager->addErrorMessage(_("Error processing the payment. Invalid card details."));

                throw new Exception(_("Error processing the payment. Invalid card details."));

                break;

            case -1:

                $this->messageManager->addErrorMessage(_("Transaction declined. Please check the provided payment details."));

                throw new Exception(_("Transaction declined. Please check the provided payment details."));


            case 2:

                $this->messageManager->addSuccessMessage(_("Payment details successfully processed."));

                break;

            default:
                
                $this->messageManager->addErrorMessage(_("Oops something went wrong. Please try again or contact support."));

                throw new Exception(_("Error processing the payment. Please try again or contact support."));

                break;
        }

    }

    /**
     * @param $body
     * @return Consumer
     */
    private function initialiseConsumer($body)
    {
        $consumer = new Consumer($body['consumer']['name'], $body['consumer']['surname'], $body['consumer']['email']);
        return $consumer;
    }

    /**
     * @param $body
     * @return Transaction
     */
    private function initialiseTransaction($body)
    {
        $transaction = new Transaction($body['transaction']['reference'], $body['transaction']['description'], $body['transaction']['currency'], $body['transaction']['amount']);
        return $transaction;
    }

    /**
     * @param $body
     * @return CreditCard
     */
    private function initialiseCreditCard($body)
    {
        $creditCard = new CreditCard($body['creditCard']['number'], $body['creditCard']['cardHolder'], $body['creditCard']['expiryYear'], $body['creditCard']['expiryMonth'], $body['creditCard']['cvv']);
        return $creditCard;
    }
}
